/***********************************
 * assignment 3                    *
 * EECS 488 - Fall 2000            *
 * Written 10/00 by Atul Nayak     *
 ***********************************/

#include "common.hxx" 
#include "wall.hxx"
#include "orb.hxx"
#include "ped.hxx"
#include "tele.hxx"
#include "space.hxx"
#include "viewer.hxx"
#include "door.hxx"
#include "start.hxx"

//#define filelength 80

/* instantiate objects here */
/****************************/

char theMaze[500][500];
int firstTime;
vector<wall> wallVector;
vector<door> doorVector;
vector<orb> orbVector;
vector<ped> pedVector;
vector<teleporter> teleporterVector;
vector<space> spaceVector;
vector<start> startVector;
viewer camera;

/*************************************************************/
/*							     */  
/* Read in the maze from the file maze.txt.		     */	
/* Store it in a 2D array. 				     */	
/* Print the maze. 					     */	
/*							     */  
/*************************************************************/
void readMaze()
{
	FILE* fp = fopen(mazefile,"r");			// open the maze file in read mode
	if (!fp) {					// exit if error in opening file
		printf("Couldn't open file\n");		
		exit(1);
	}
	
	int i=0;					// variables to access the maze array.
	int j=0;					// i is the cartesian x direction
	while(!feof(fp))				// j is the cartesian y direction
	{
		char tmp;
		fscanf(fp,"%c",&tmp);			// read in the file
		if (tmp == '\n')			
		{
			i++;
			j=0;
		}
		else
		{
			theMaze[i][j] = tmp;
			j++;		    	
		}
	}
	
	for (i=0;i<10;i++)				// print the  maze
	{	
		for (int j=0;j<20;j++)
			printf("%c",theMaze[i][j]);
		printf("\n");
	}
}

/*************************************************************/
/*							     */  
/* At every instant we have to calculate the distance of     */	
/* each object in the maze so that objects close to the	     */	
/* camera can be brighter than those away from the camera.   */	
/* This function returns the distance of an object	     */
/* from the camera given it's location in the maze.	     */  
/*************************************************************/
float calculateDistance(int currentX, int currentY)
{
	float viewerX = camera.getEyesX();		//get the camera's location
	float viewerY = camera.getEyesY();

	float diffX = viewerX - ((currentX * 5.0) + 2.5);	// calculate the difference in coordinates
	float diffY = viewerY - ((currentY * 5.0) + 2.5);

	float distance = sqrt(pow(diffX, 2.0) + pow(diffY, 2.0));

	return distance;
}

void drawNothing(int x, int y)
{

}

void drawFloor(int x, int y)
{
	if (firstTime)
	{
		space spaceObj;
		spaceObj.drawSpace(x, y);
		spaceVector.push_back(spaceObj);
	}
	else
	{
		for (vector<space>::iterator spaceCounter = spaceVector.begin(); spaceCounter != spaceVector.end(); spaceCounter++)
			if (((*spaceCounter).getPositionX() == x) && ((*spaceCounter).getPositionY() == y))
			{
				float distanceFromEyes = calculateDistance(x, y);
				if (distanceFromEyes <= 10.0)
					(*spaceCounter).brightestColor();
				else
					if (distanceFromEyes >= 110.0)		
						(*spaceCounter).darkestColor();		
					else
						(*spaceCounter).changeColor(1.0 - ((distanceFromEyes - 10.0) * 0.01));
				(*spaceCounter).drawSpace(x,y);
			}
	}
	
}

void drawDoor(int x, int y)
{
	if (firstTime)
	{
		door doorObj;
		if ((theMaze[x-1][y+1] == ' ') || (theMaze[x+1][y+1] == ' ') || (theMaze[x+1][y-1] == ' ') || (theMaze[x-1][y-1] == ' '))
		doorObj.setAngle();
		doorObj.drawDoor(x, y);
		doorVector.push_back(doorObj);
		
		if ((theMaze[x-1][y+1] == ' ') || (theMaze[x+1][y+1] == ' ') || (theMaze[x+1][y-1] == ' ') || (theMaze[x-1][y-1] == ' '))
		{
			wall wallObj;
			wallObj.setWallIsDoor();
			wallObj.drawWall(x,y);
			wallObj.darkestColor();	
			wallVector.push_back(wallObj);
		}	

	}
	else
	{
		for (vector<door>::iterator doorCounter = doorVector.begin(); doorCounter != doorVector.end(); doorCounter++)
			if (((*doorCounter).getPositionX() == x) && ((*doorCounter).getPositionY() == y))
			{
				float distanceFromEyes = calculateDistance(x, y);
				if (distanceFromEyes <= 10.0)
					(*doorCounter).brightestColor();
				else
					if (distanceFromEyes >= 110.0)		
						(*doorCounter).darkestColor();		
					else
						(*doorCounter).changeColor(1.0 - ((distanceFromEyes - 10.0) * 0.01));			
				(*doorCounter).drawDoor(x,y);
			}
	}
	
}


void drawOrb(int x, int y)
{
	if (firstTime)
	{
		orb orbObj;
		orbObj.drawOrb(x, y);
		orbVector.push_back(orbObj);
	}
	else
	{
		for (vector<orb>::iterator orbCounter = orbVector.begin(); orbCounter != orbVector.end(); orbCounter++)
			if (((*orbCounter).getPositionX() == x)
 && ((*orbCounter).getPositionY() == y))
			{
				float distanceFromEyes = calculateDistance(x, y);
				if (distanceFromEyes <= 10.0)
					(*orbCounter).brightestColor();
				else
					if (distanceFromEyes >= 110.0)		
						(*orbCounter).darkestColor();		
					else
						(*orbCounter).changeColor(1.0 - ((distanceFromEyes - 10.0) * 0.01));
				(*orbCounter).drawOrb(x,y);
			}
	}

}

void drawPedestal(int x, int y)
{
	if (firstTime)
	{
		ped pedObj;
		pedObj.drawPed(x, y);
		pedVector.push_back(pedObj);
	}
	else
	{
		for (vector<ped>::iterator pedCounter = pedVector.begin(); pedCounter != pedVector.end(); pedCounter++)
			if (((*pedCounter).getPositionX() == x) && ((*pedCounter).getPositionY() == y))
			{
				float distanceFromEyes = calculateDistance(x, y);
				if (distanceFromEyes <= 10.0)
					(*pedCounter).brightestColor();
				else
					if (distanceFromEyes >= 110.0)		
						(*pedCounter).darkestColor();		
					else
						(*pedCounter).changeColor(1.0 - ((distanceFromEyes - 10.0) * 0.01));								(*pedCounter).drawPed(x,y);
			}
	}
}

void drawTeleporter(int x, int y)
{
	if (firstTime)
	{
		teleporter teleporterObj;
		teleporterObj.drawTeleporter(x, y);
		teleporterVector.push_back(teleporterObj);
	}
	else
	{
		for (vector<teleporter>::iterator teleporterCounter = teleporterVector.begin(); teleporterCounter != teleporterVector.end(); teleporterCounter++)
			if (((*teleporterCounter).getPositionX() == x) && ((*teleporterCounter).getPositionY() == y))
			{	
				float distanceFromEyes = calculateDistance(x, y);
//				printf("x is %d y is % d\n ",x,y);				
//				printf("Distance of teleporter is %.2f\n", distanceFromEyes);
		if (distanceFromEyes <= 10.0)
					(*teleporterCounter).brightestColor();
				else
					if (distanceFromEyes >= 110.0)		
						(*teleporterCounter).darkestColor();		
					else
						(*teleporterCounter).changeColor(1.0 - ((distanceFromEyes - 10.0) * 0.01));							(*teleporterCounter).drawTeleporter(x,y);
			}
	}
}

void startHere(int x, int y)
{
/*	if (firstTime)
	{

	}
	glPushMatrix();
		glTranslatef((startX*5.0)+2.5,(startY*5.0)+2.5,0);
		glColor3f(0.0,1.0,0.0);
		glBegin(GL_LINE_LOOP);
			glVertex3f(-2.5,2.5,0.0);
			glVertex3f(2.5,-2.5,0.0);
		glEnd();	
		glBegin(GL_LINE_LOOP);
			glVertex3f(2.5,2.5,0.0);
			glVertex3f(-2.5,-2.5,0.0);
		glEnd();	
	glPopMatrix();
*/
	if (firstTime)
	{
		camera.initialize(x, y);
		start startObj;
		startObj.drawStart(x, y);
		startVector.push_back(startObj);
	}
	else
	{
		for (vector<start>::iterator startCounter = startVector.begin(); startCounter != startVector.end(); startCounter++)
			if (((*startCounter).getPositionX() == x) && ((*startCounter).getPositionY() == y))
			{
				float distanceFromEyes = calculateDistance(x, y);				
				printf("distance %.2f\n", distanceFromEyes);	
				if (distanceFromEyes <= 10.0)
					(*startCounter).brightestColor();
				else
					if (distanceFromEyes >= 110.0)		
						(*startCounter).darkestColor();		
					else
						(*startCounter).changeColor(1.0 - ((distanceFromEyes - 10.0) * 0.01));								(*startCounter).drawStart(x,y);
			}
	}


}




/*************************************************************/
/*							     */  
/* Draw the Wall. The walls within 10 feet of the camera     */	
/* will appear brighetr than those beyond 10 feet. If the    */	
/* wall is 110 feet away , it will be completely dark.	     */ 
/*							     */  	
/*************************************************************/
void drawWall(int x, int y)
{
	if (firstTime)
	{
		wall wallObj;
		wallObj.drawWall(x, y);
		wallVector.push_back(wallObj);
	}
	else
	{
		for (vector<wall>::iterator wallCounter = wallVector.begin(); wallCounter != wallVector.end(); wallCounter++)
			if (((*wallCounter).getPositionX() == x) && ((*wallCounter).getPositionY() == y))
			{
				float distanceFromEyes = calculateDistance(x, y);
//				printf("Distance is %.2f\n", distanceFromEyes);
				if (distanceFromEyes <= 10.0)
					(*wallCounter).brightestColor();
				else
					if (distanceFromEyes >= 110.0)		
						(*wallCounter).darkestColor();		
					else
						(*wallCounter).changeColor(1.0 - ((distanceFromEyes - 10.0) * 0.01));			
				(*wallCounter).drawWall(x,y);
			}
	}
	
}


void collideWithOrb(int currentX, int currentY)
{
	for (vector<orb>::iterator orbCounter = orbVector.begin(); orbCounter != orbVector.end(); orbCounter++)
	{
		if (((*orbCounter).getPositionX() == currentX)
 && ((*orbCounter).getPositionY() == currentY))
		{
			(*orbCounter).setOrbTaken();
			(*orbCounter).darkestColor();
			for (vector<ped>::iterator pedCounter = pedVector.begin(); pedCounter != pedVector.end(); pedCounter++)
				if (((*pedCounter).getPositionX() == currentX)
 && ((*pedCounter).getPositionY() == currentY))
				{
					(*pedCounter).setOrbTaken();
					(*pedCounter).makeRed();		
				}
		}
	}
}

void collideWithTeleporter(int currentX, int currentY)
{
	int i=0;
	int j=0;
	int done = FALSE;
	while(!done) 
	{
		i = 99*rand()/RAND_MAX;
		j = 99*rand()/RAND_MAX;
//		printf("i %d j %d \n", i, j);
		if (( i == currentX) && (j == currentY))
		{
			while (i == currentX)
				i = rand()/RAND_MAX;
			while (j == currentX)
				j = rand()/RAND_MAX;
		}
		if (theMaze[i][j] == '.')
		{	
			camera.changeEyes(i,j);
			done = TRUE;
		}
	}	
} 

void collideWithAllWalls()
{
	float oldEyesX = camera.getCurrentEyesX();
	float oldEyesY = camera.getCurrentEyesY();
	float newEyesX = camera.getNextEyesX();
	float newEyesY = camera.getNextEyesY();

	int oX = oldEyesX/5.0;
	int oY = oldEyesY/5.0;
	int nX = newEyesX/5.0;
	int nY = newEyesY/5.0;

	int oldXnewY = FALSE;
	int newXoldY = FALSE;

	for (vector<wall>::iterator wallCounter = wallVector.begin(); wallCounter != wallVector.end(); wallCounter++)
	{
		if ((newEyesX <= (*wallCounter).getRightEdge()) && 
		    (newEyesX >= (*wallCounter).getLeftEdge())  &&
		    (newEyesY >= (*wallCounter).getBottomEdge()) &&	 
		    (newEyesY <= (*wallCounter).getTopEdge()))
			{
				printf("newEyesX %.2f newEyesY %.2f\n", newEyesX, newEyesY);
				printf("R %.2f L %.2f T %.2f B %.2f\n",(*wallCounter).getRightEdge(), (*wallCounter).getLeftEdge(), (*wallCounter).getTopEdge(), (*wallCounter).getBottomEdge());	
				if (theMaze[oX][nY] != '#')
					oldXnewY = TRUE;
				if (theMaze[nX][oY] != '#')
					newXoldY = TRUE;
				if ((theMaze[oX][nY] == '#') && (theMaze[nX][oY] == '#'))
				{
					oldXnewY = TRUE;
					newXoldY = TRUE;
				}			
			}
	}

	if ((oldXnewY) && (newXoldY)) 
		camera.oldXoldY();				
	else
	{
		if (oldXnewY) camera.oldXnewY();
		if (newXoldY) camera.newXoldY();
	}
}



/*************************************************************/
/* keyboard						     */	
/*							     */  
/* deal with the keyboard input to the program.  	     */	
/* 							     */	
/*************************************************************/

void keyboard(unsigned char userkey, int x, int y)
{

	switch(userkey)
	{
		case 'i':
			camera.moveForward();

			int currentX = camera.getCurrentPositionX();
			int currentY = camera.getCurrentPositionY();	
			int nextX = camera.getNextPositionX();
			int nextY = camera.getNextPositionY();	

			collideWithAllWalls();

			char nextObject = theMaze[nextX][nextY];

			switch (nextObject)
			{
//				case '#':
//					collideWithWall(currentX, currentY, nextX, nextY);
//					break;
				case 'O' : 
					collideWithOrb(currentX, currentY);
					break;
				case 'T':
					collideWithTeleporter(currentX, currentY);
					break;
				default:
					camera.resetSlideAlongX();
					camera.resetSlideAlongY();
					break;
			}

			break;
		case 'j': 
			camera.turnLeft();
			camera.resetSlideAlongX();
			camera.resetSlideAlongY();
			break;
		case 'l':
			camera.turnRight();
			camera.resetSlideAlongX();
			camera.resetSlideAlongY();
			break;
		default:
			break;	
	}
}

/*************************************************************
 *
 * update
 * 
 * update everything that needs to be updated in the scene - this
 * function should be called via a timer to ensure that updates 
 * happen at a regular rate, independent of the graphics update
 * rate
 *
 *************************************************************/

void update(int value)
    {
   
    /* at the end of this function
       reinstate the timer callback */
    /********************************/



    glutTimerFunc(5, update, 1);
    } 

void chooseWhatToDraw(char c, int centreX, int centreY)
{

	switch(c)
	{
	case ' ':
		drawNothing(centreX, centreY);
		break;
	case '#':
		{
		drawWall(centreX, centreY);
		break;		
		}
	case '.':
		drawFloor(centreX, centreY);
		break;
	case 'D':
		drawDoor(centreX, centreY);
		break;		
	case 'O':
		drawOrb(centreX, centreY);
		drawPedestal(centreX, centreY);
		break;		
	case 'T':
		drawTeleporter(centreX, centreY);
		break;		
	case 'X':
		startHere(centreX, centreY);
		break;		
	case 'P':
		drawPedestal(centreX, centreY);
		break;
	default :
		break;
	}
}	


/*************************************************************
 *
 * display
 * 
 * this function is called each time the screen is to be drawn
 *************************************************************/
void display(void)
{

    /* clear the screen to the clear colour */
    /****************************************/

	glClear(GL_COLOR_BUFFER_BIT);
//	glColor3f(1.0,1.0,1.0);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60, 1, 0.1, 110);
	camera.lookAt();

	for (int i = 0; i < 80; i++)
		for (int j = 0; j < 80; j++)
			chooseWhatToDraw(theMaze[i][j], i, j);
	firstTime = FALSE;
    /* draw everything */
    /*******************/


    /* swap buffers */
    /****************/
  
    glutSwapBuffers();
}

/*************************************************************
 *
 * dealWithParameters
 * 
 * code to deal with the command-line
 * parameters for the program
 *************************************************************/
 
void dealWithParameters(int argc, char **argv)
{

}



/*************************************************************
 *
 * main
 * 
 *************************************************************/

int main(int argc, char **argv)
    {

    /* deal with any GLUT command Line options */
    /*******************************************/
        
    glutInit(&argc, argv);
    
    /* deal with any command line options for this program */
    /*******************************************************/

    dealWithParameters(argc, argv);

	srand((unsigned)time(NULL));
    /* create an output window */
    /***************************/

    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
    glutInitWindowSize(500, 500);
    glutCreateWindow("Berzoik");
    if (glutGet(GLUT_WINDOW_COLORMAP_SIZE) != 0)
	{
	printf("FAIL: bad RGBA colormap size\n");
	exit(1);
	}

	readMaze();				
	firstTime = TRUE;

    /* set up for accepting keyboard commands */
    /******************************************/

    glutKeyboardFunc(keyboard);

    /* set up the logical graphics space */
    /*************************************/
    
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(60, 1, 0.1, 110);
    glClearColor(0.0, 0.0, 0.0, 1.0);
    glMatrixMode(GL_MODELVIEW);

    /* assign the display function */
    /*******************************/

    glutDisplayFunc(display);

    /* assign the idle function */
    /****************************/

    glutIdleFunc(display);

    /* set up a timer to regularly update the scene */
    /************************************************/

    glutTimerFunc(5, update, 1);

    /* set everything going */
    /************************/
    
    glutMainLoop();
 
  return 0;  

}

