/***********************************
 * assignment 3                    *
 * EECS 488 - Fall 2000            *
 * Written 11/00 by Atul Nayak     *
 ***********************************/

#include "common.hxx" 			// a common header file
#include "wall.hxx"			// the wall header
#include "orb.hxx"			// the orb header
#include "ped.hxx"			// the pedestal header
#include "tele.hxx"			// the teleporter header
#include "space.hxx"			// the space header
#include "viewer.hxx"			// the viewer header
#include "door.hxx"			// the door header
#include "start.hxx"			// teh start header


/* instantiate objects here */
/****************************/

char theMaze[MAX_LENGTH][MAX_LENGTH];			// the maze
struct							// a structure of flags	
{
	int firstTime;					// indicates first run of the program
	int moveForward;				// indicates that 'i' key was pressed
	int turnLeft;					// indicates that 'j' key was pressed
	int turnRight;					// indicates that 'l' key was pressed
}flags;

//vectors of different classes			
vector<wall> wallVector;			
vector<door> doorVector;
vector<orb> orbVector;
vector<ped> pedVector;
vector<teleporter> teleporterVector;
vector<space> spaceVector;
vector<start> startVector;

// the camera
viewer camera;			

/*************************************************************/
/*							     */  
/* Read in the maze from the file maze.txt.		     */	
/* Store it in a 2D array. 				     */	
/*							     */  
/*************************************************************/
void readMaze()
{
	FILE* fp = fopen(mazefile,"r");			// open the maze file in read mode
	if (!fp) {					// exit if error in opening file
		printf("Couldn't open file\n");		
		exit(1);
	}
	
	int i=0;					// variables to access the maze array.
	int j=0;					// i is the cartesian x direction
	while(!feof(fp))				// j is the cartesian y direction
	{
		char tmp;
		fscanf(fp,"%c",&tmp);			// read in the file
		if (tmp == '\n')			
		{
			i++;
			j=0;
		}
		else
		{
			theMaze[i][j] = tmp;
			j++;		    	
		}
	}
}

/*************************************************************/
/*							     */  
/* At every instant we have to calculate the distance of     */	
/* each object in the maze so that objects close to the	     */	
/* camera can be brighter than those away from the camera.   */	
/* This function returns the distance of an object	     */
/* from the camera given it's location in the maze.	     */  
/*************************************************************/
float calculateDistance(int currentX, int currentY)
{
	float viewerX = camera.getEyesX();		//get the camera's location
	float viewerY = camera.getEyesY();

	float diffX = viewerX - ((currentX * 5.0) + 2.5);	// calculate the difference in coordinates
	float diffY = viewerY - ((currentY * 5.0) + 2.5);

	float distance = sqrt((diffX * diffX) + (diffY * diffY));

	return distance;
}

/*************************************************************/
/*							     */  
/* Draw the floor. The floor within 10 feet of the	     */
/* camera will appear brighter than those beyond 10 feet. If */	
/* the door is 110 feet away,it will be completely dark.     */ 
/* 							     */	
/*************************************************************/
void drawFloor(int x, int y)
{
	if (flags.firstTime)
	{
		space spaceObj;			// create floor (space) object
		spaceObj.drawSpace(x, y);	// draw it
		spaceVector.push_back(spaceObj);// store it
	}
	else
	{	// for all the spaces
		for (vector<space>::iterator spaceCounter = spaceVector.begin(); spaceCounter != spaceVector.end(); spaceCounter++)
			if (((*spaceCounter).getPositionX() == x) && ((*spaceCounter).getPositionY() == y))
			{
				float distanceFromEyes = calculateDistance(x, y); // calculate the distance from the camera
				if (distanceFromEyes <= 10.0)			  // brighten or darken the floor	
					(*spaceCounter).brightestColor();
				else
					if (distanceFromEyes >= 110.0)		
						(*spaceCounter).darkestColor();		
					else
						(*spaceCounter).changeColor(1.0 - ((distanceFromEyes - 10.0) * 0.01));
				(*spaceCounter).drawSpace(x,y);			// draw floor
			}
	}
}

/*************************************************************/
/*							     */  
/* Draw the door. The doors within 10 feet of the	     */
/* camera will appear brighter than those beyond 10 feet. If */	
/* the door is 110 feet away,it will be completely dark.     */ 
/* If the door is on the boundary, check that the viewer does*/
/* not slip out of the maze. Do this by drawing a transparent*/  	
/* wall where the door is located.			     */
/*************************************************************/
void drawDoor(int x, int y)
{
	if (flags.firstTime)
	{
		door doorObj;			// door object
		if ((theMaze[x-1][y+1] == ' ') || (theMaze[x+1][y+1] == ' ') || (theMaze[x+1][y-1] == ' ') || (theMaze[x-1][y-1] == ' '))		
			doorObj.setAngle();	// if the door is on the periphery rotate it by 90 degrees.
		doorObj.drawDoor(x, y);		// draw the door
		doorVector.push_back(doorObj);	// store it
		
		// Check if the door is on the boundary, ie. check if there are spaces around the door.
		// if yes then create a transparent wall on the position.
		if ((theMaze[x-1][y+1] == ' ') || (theMaze[x+1][y+1] == ' ') || (theMaze[x+1][y-1] == ' ') || (theMaze[x-1][y-1] == ' '))
		{
			wall wallObj;		// create a wall
//			wallObj.setWallIsDoor();// set a flag to indicate that the wall is actually a door that does not open!
			wallObj.drawWall(x,y);	// draw the wall
			wallObj.darkestColor();	// darken it 
			wallVector.push_back(wallObj);// store it
		}	

	}
	else
	{	// for all the doors
		for (vector<door>::iterator doorCounter = doorVector.begin(); doorCounter != doorVector.end(); doorCounter++)
			if (((*doorCounter).getPositionX() == x) && ((*doorCounter).getPositionY() == y))
			{
				float distanceFromEyes = calculateDistance(x, y); //calculate the distance from the camera	
				if (distanceFromEyes <= 10.0)			  // brigthen or darken the object	
					(*doorCounter).brightestColor();
				else
					if (distanceFromEyes >= 110.0)		
						(*doorCounter).darkestColor();		
					else
						(*doorCounter).changeColor(1.0 - ((distanceFromEyes - 10.0) * 0.01));			
				(*doorCounter).drawDoor(x,y);			// draw the door
			}
	}
}

/*************************************************************/
/*							     */  
/* Draw the orb. The orbs within 10 feet of the		     */
/* camera will appear brighter than those beyond 10 feet. If */	
/* the orb is 110 feet away,it will be completely dark	     */ 
/*							     */  	
/*************************************************************/
void drawOrb(int x, int y)
{
	if (flags.firstTime)
	{
		orb orbObj;			// an orb object
		orbObj.drawOrb(x, y);		//draw it
		orbVector.push_back(orbObj);	//store it
	}
	else
	{
		for (vector<orb>::iterator orbCounter = orbVector.begin(); orbCounter != orbVector.end(); orbCounter++)
			if (((*orbCounter).getPositionX() == x) && ((*orbCounter).getPositionY() == y))
			{
				float distanceFromEyes = calculateDistance(x, y);//calculate the distance from the camera
				if (distanceFromEyes <= 10.0)			// brighten or darken the object
					(*orbCounter).brightestColor();
				else
					if (distanceFromEyes >= 110.0)		
						(*orbCounter).darkestColor();		
					else
						(*orbCounter).changeColor(1.0 - ((distanceFromEyes - 10.0) * 0.01));
				(*orbCounter).drawOrb(x,y);			// draw the orb
			}
	}

}

/*************************************************************/
/*							     */  
/* Draw the pedestal. The pedestals within 10 feet of the    */
/* camera will appear brighter than those beyond 10 feet. If */	
/* the pedestal is 110 feet away,it will be completely dark  */ 
/*							     */  	
/*************************************************************/
void drawPedestal(char object, int x, int y)
{
	if (flags.firstTime)
	{
		ped pedObj;			// a pedestal object
		pedObj.drawPed(x, y);		// draw the pedestal
		if (object == 'P')		// if only a pedestal is to be drawn then 
		{
			pedObj.setOrbTaken();   // make it red as if an orb was taken away
			pedObj.makeRed();
		}
		pedVector.push_back(pedObj);	// store it
	}
	else
	{	// check all the pedestals
		for (vector<ped>::iterator pedCounter = pedVector.begin(); pedCounter != pedVector.end(); pedCounter++)
			if (((*pedCounter).getPositionX() == x) && ((*pedCounter).getPositionY() == y))
			{
				float distanceFromEyes = calculateDistance(x, y);	//calculate the distance from the camera
				if (distanceFromEyes <= 10.0)				// brighten or darken the object
					(*pedCounter).brightestColor();
				else
					if (distanceFromEyes >= 110.0)		
						(*pedCounter).darkestColor();		
					else
						(*pedCounter).changeColor(1.0 - ((distanceFromEyes - 10.0) * 0.01));								(*pedCounter).drawPed(x,y);				// draw the pedestal
			}
	}
}

/*************************************************************/
/*							     */  
/* Draw the teleporter. The teleporters within 10 feet of the*/
/* camera will appear brighter than those beyond 10 feet. If */	
/* the teleporter is 110 feet away,it will be completely dark*/ 
/*							     */  	
/*************************************************************/
void drawTeleporter(int x, int y)
{
	if (flags.firstTime)
	{
		teleporter teleporterObj;			// a teleporter object
		teleporterObj.drawTeleporter(x, y);		// draw Teleporter 
		teleporterVector.push_back(teleporterObj);	// store it
	}
	else
	{	// for all the teleporters
		for (vector<teleporter>::iterator teleporterCounter = teleporterVector.begin(); teleporterCounter != teleporterVector.end(); teleporterCounter++)
			if (((*teleporterCounter).getPositionX() == x) && ((*teleporterCounter).getPositionY() == y))
			{	
				float distanceFromEyes = calculateDistance(x, y); // calculate distance from the camera
				if (distanceFromEyes <= 10.0)			  // brighten or darken the teleporter 	
					(*teleporterCounter).brightestColor();
				else
					if (distanceFromEyes >= 110.0)		
						(*teleporterCounter).darkestColor();		
					else
						(*teleporterCounter).changeColor(1.0 - ((distanceFromEyes - 10.0) * 0.01));							(*teleporterCounter).drawTeleporter(x,y);	  // draw the teleporter
			}
	}
}
/*************************************************************/
/*							     */  
/* Draw the Starting Position. Initialize the camera position*/
/*							     */  	
/*************************************************************/

void startHere(int x, int y)
{
	if (flags.firstTime)				
	{
		camera.initialize(x, y);	// place the camera at this position
		start startObj;			// draw the starting position and 
		startObj.drawStart(x, y);	// store it in a vector	
		startVector.push_back(startObj);
	}
	else
	{	// for all objects of the start class
		for (vector<start>::iterator startCounter = startVector.begin(); startCounter != startVector.end(); startCounter++)
			if (((*startCounter).getPositionX() == x) && ((*startCounter).getPositionY() == y))
			{
				float distanceFromEyes = calculateDistance(x, y); //calculate the distance				
				if (distanceFromEyes <= 10.0)			  // brighten or darken the object
					(*startCounter).brightestColor();
				else
					if (distanceFromEyes >= 110.0)		
						(*startCounter).darkestColor();		
					else
						(*startCounter).changeColor(1.0 - ((distanceFromEyes - 10.0) * 0.01));								(*startCounter).drawStart(x,y);			// draw the start position
			}
	}
}


/*************************************************************/
/*							     */  
/* Draw the Wall. The walls within 10 feet of the camera     */	
/* will appear brighter than those beyond 10 feet. If the    */	
/* wall is 110 feet away , it will be completely dark.	     */ 
/*							     */  	
/*************************************************************/
void drawWall(int x, int y)
{
	if (flags.firstTime)
	{
		wall wallObj;
		wallObj.drawWall(x, y);
		wallVector.push_back(wallObj);
	}
	else
	{
		for (vector<wall>::iterator wallCounter = wallVector.begin(); wallCounter != wallVector.end(); wallCounter++)
			if (((*wallCounter).getPositionX() == x) && ((*wallCounter).getPositionY() == y))
			{
				float distanceFromEyes = calculateDistance(x, y);
				if (distanceFromEyes <= 10.0)
					(*wallCounter).brightestColor();
				else
					if (distanceFromEyes >= 110.0)		
						(*wallCounter).darkestColor();		
					else
						(*wallCounter).changeColor(1.0 - ((distanceFromEyes - 10.0) * 0.01));			
				(*wallCounter).drawWall(x,y);
			}
	}
	
}

/*************************************************************/
/*							     */  
/* Check if the viewer has walked over an orb.	             */	
/* The orb will disappear when collected.		     */	 
/*							     */  	
/*************************************************************/
void collideWithOrb(int currentX, int currentY)
{
	// check all the orbs
	for (vector<orb>::iterator orbCounter = orbVector.begin(); orbCounter != orbVector.end(); orbCounter++)
	{
		// if an orb position matches the current position of the viewer
		if (((*orbCounter).getPositionX() == currentX) && ((*orbCounter).getPositionY() == currentY))
		{
			(*orbCounter).setOrbTaken();		// the orb is taken
			(*orbCounter).darkestColor();		// it disappears
	
			// locate the pedestal drawn on the same spot		
			for (vector<ped>::iterator pedCounter = pedVector.begin(); pedCounter != pedVector.end(); pedCounter++)
				if (((*pedCounter).getPositionX() == currentX) && ((*pedCounter).getPositionY() == currentY))
				{
					(*pedCounter).setOrbTaken();	// since orb is taken 
					(*pedCounter).makeRed();	// make the pedestal red in color.	
				}
		}
	}
	if ((currentX == 0) || (currentY == 0))
		camera.oldXoldY();
}


/*************************************************************/
/*							     */  
/* Check if the viewer has walked over a teleporter.         */	
/* If yes, you should get transported to a safe free position*/
/*in the maze.						     */	 
/*							     */  	
/*************************************************************/
void collideWithTeleporter(int currentX, int currentY)
{
	int i=0;				// temporary variables to store a random position
	int j=0;
	int done = FALSE;			// flag to indicate that a safe position was found
	while(!done) 
	{
		i = 99*rand()/RAND_MAX;		// get a random number
		j = 99*rand()/RAND_MAX;		// get a random number
		if (( i == currentX) && (j == currentY))	// make sure it is not the current position
		{
			while (i == currentX)			// if yes tehn calculate another random number
				i = rand()/RAND_MAX;
			while (j == currentX)
				j = rand()/RAND_MAX;
		}	
		if (theMaze[i][j] == '.')	// check if it is a floor space,else look for another place
		{	
			camera.changeEyes(i,j);	// change the camera's position
			done = TRUE;		// safe position was found.
		}
	}	
} 

/*************************************************************/
/*							     */  
/* Check if the viewer has collides with a wall              */	
/* If yes, you should slide along the wall,until a corner    */
/* is reached.						     */	 
/*							     */  	
/*************************************************************/
void collideWithAllWalls()
{
	float oldEyesX = camera.getCurrentEyesX();	// get the previous and next position 
	float oldEyesY = camera.getCurrentEyesY();      // of the camera.
	float newEyesX = camera.getNextEyesX();
	float newEyesY = camera.getNextEyesY();

	int oX = oldEyesX/5.0;				// calculate the previous and next position 
	int oY = oldEyesY/5.0;				// of the camera in the maze ie. get it's
	int nX = newEyesX/5.0;				// location in the 2D array.
	int nY = newEyesY/5.0;

	int oldXnewY = FALSE;				// set flags to false
	int newXoldY = FALSE;

	// check all the walls
	for (vector<wall>::iterator wallCounter = wallVector.begin(); wallCounter != wallVector.end(); wallCounter++)
	{
		// if the next position collides with any wall in the maze
		if ((newEyesX <= (*wallCounter).getRightEdge()) && 
		    (newEyesX >= (*wallCounter).getLeftEdge())  &&
		    (newEyesY >= (*wallCounter).getBottomEdge()) &&	 
		    (newEyesY <= (*wallCounter).getTopEdge()))
			{
				// check if there is a wall in the X direction	
				// if yes then set oldXnewY to true
				// so that you slide along the wall in the Y direction
				if (theMaze[oX][nY] != '#')
					oldXnewY = TRUE;

				// check if there is a wall in the Y direction	
				// if yes then set newXoldY to true
				// so that you slide along the wall in the X direction
				if (theMaze[nX][oY] != '#')
					newXoldY = TRUE;

				// if you have reached a corner stay put
				// set both flags
				if ((theMaze[oX][nY] == '#') && (theMaze[nX][oY] == '#'))
				{
					oldXnewY = TRUE;
					newXoldY = TRUE;
				}			
			}
	}

	if ((oldXnewY) && (newXoldY)) 	// if both flags are set
		camera.oldXoldY();	// go back to previous position	before collision		
	else
	{
		if (oldXnewY) camera.oldXnewY();	// move in the direction set 
		if (newXoldY) camera.newXoldY();
	}
}

/*************************************************************/
/* keyboard						     */	
/*							     */  
/* deal with the keyboard input to the program.  	     */	
/* 							     */	
/*************************************************************/

void keyboard(unsigned char userkey, int x, int y)
{
	// set appropriate flag
	switch(userkey)
	{
		case 27:
			exit(-1);
			break;
		case 'i':
			flags.moveForward = TRUE;	
			break;
		case 'j': 
			flags.turnLeft = TRUE;
			break;
		case 'l':
			flags.turnRight = TRUE;
			break;
		default:
			break;	
	}
}

/*************************************************************
 *
 * update
 * 
 * update everything that needs to be updated in the scene - this
 * function should be called via a timer to ensure that updates 
 * happen at a regular rate, independent of the graphics update
 * rate
 *
 *************************************************************/

void update(int value)
    {
   
    /* at the end of this function
       reinstate the timer callback */
    /********************************/

	// i key was pressed
	if (flags.moveForward)
	{
		camera.moveForward();				// calculate the new position of the camera		
		int currentX = camera.getCurrentPositionX();	// obtain the previous position
		int currentY = camera.getCurrentPositionY();	// and the next position of the camera in the array
		int nextX = camera.getNextPositionX();
		int nextY = camera.getNextPositionY();	
		collideWithAllWalls();				// check if there are any collisions with the wallls
		char nextObject = theMaze[nextX][nextY];	// find out what the object in the next position is
		switch (nextObject)					
		{
			case 'O' : 				// if the next object is an Orb 
				collideWithOrb(currentX, currentY);
				break;
			case 'T':				// if the next object is a Teleporter
				collideWithTeleporter(currentX, currentY);
				break;
			case 'D':				// if there is a door situated on the periphery
				if ((currentX == 0) || (currentY == 0))
					camera.oldXoldY();	// stay at previous position
				break;
			case 'P':
				if ((currentX == 0) || (currentY == 0)) // if there is a pedestal on the periphery
					camera.oldXoldY();		//  stay at previous position
				break;
			case ' ':				// in case an orb or a door is on the periphery
				camera.oldXoldY();		// the next position will be a space 
				break;				// so just move back to the previous position
			default:
				break;
		}
		flags.moveForward = FALSE;			// reset the flag
	}
	
	if (flags.turnLeft)					// if j key was pressed
	{	
		flags.turnRight = FALSE;			// reset the turnRight flag
		camera.turnLeft();				// turn Left
		flags.turnLeft = FALSE;				// reset the turnLeft flag
	}	

	if (flags.turnRight)
	{
		flags.turnLeft = FALSE;				// reset the turnLeft flag
		camera.turnRight();				// turn Right
		flags.turnRight = FALSE;			// reset the turnRight flag
	}

	glutTimerFunc(5, update, 1);
    } 


/****************************************************************/
/*  Depending on what the Maze array character is call the      */
/*  appropriate draw function					*/
/*  Pass the location of the object in the array to the function*/
/****************************************************************/
void chooseWhatToDraw(char c, int centreX, int centreY)
{

	switch(c)
	{
	case ' ':				// a space so don't draw anything
		break;
	case '.':
		drawFloor(centreX, centreY);	// draw a floor space
		break;
	case 'D':
		drawFloor(centreX, centreY);	// draw a floor space	
		drawDoor(centreX, centreY);	// draw a door
		break;		
	case 'O':
		drawFloor(centreX, centreY);	// draw a floor space
		drawOrb(centreX, centreY);	// draw an Orb 
		drawPedestal(c, centreX, centreY); // draw a pedestal also on the same position
		break;		
	case 'T':
		drawFloor(centreX, centreY);	// draw a floor space
		drawTeleporter(centreX, centreY);// draw a teleporter
		break;		
	case 'X':
		drawFloor(centreX, centreY);	// draw a floor space
		startHere(centreX, centreY);	// draw the starting position
		break;		
	case 'P':
		drawFloor(centreX, centreY);	// draw a floor space
		drawPedestal(c, centreX, centreY);	// draw a pedestal
		break;
	default :
		break;
	}
}	


/*************************************************************
 *
 * display
 * 
 * this function is called each time the screen is to be drawn
 *************************************************************/
void display(void)
{

    /* clear the screen to the clear colour */
    /****************************************/

	glClear(GL_COLOR_BUFFER_BIT);

    /* Load the projection matrix and decide the field of vision */
    /*************************************************************/
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60, 1, 0.1, 110);

    /* position the camera		*/
    /************************************/
	camera.lookAt();

    /* draw everything */
    /*******************/

	// draw all the walls before you draw the other objects
	// otherwise the lines of other objects are not rendered correctly

	for (int i = 0; i < MAX_LENGTH; i++)
		for (int j = 0; j < MAX_LENGTH; j++)
			if (theMaze[i][j] == '#')
				drawWall(i, j);	// draw a wall

	// now draw the other objects
	for ( i = 0; i < MAX_LENGTH; i++)
		for (int j = 0; j < MAX_LENGTH; j++)
			chooseWhatToDraw(theMaze[i][j], i, j);
	flags.firstTime = FALSE;		// after everything is drawn once reset the flag.


    /* swap buffers */
    /****************/
  
    glutSwapBuffers();
}


/*************************************************************
 *
 * main
 * 
 *************************************************************/

int main(int argc, char **argv)
    {

    /* deal with any GLUT command Line options */
    /*******************************************/
        
	glutInit(&argc, argv);
    
	srand((unsigned)time(NULL)); // seed the srand function
    /* create an output window */
    /***************************/
	
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
	glutInitWindowSize(500, 500);
	glutCreateWindow("Berzoik");
	if (glutGet(GLUT_WINDOW_COLORMAP_SIZE) != 0)
	{
		printf("FAIL: bad RGBA colormap size\n");
		exit(1);
	}

	readMaze();			// read the maze	
	flags.firstTime = TRUE;		// set approriate flags
	flags.moveForward = FALSE;
	flags.turnLeft = FALSE;
	flags.turnRight = FALSE;

    /* set up for accepting keyboard commands */
    /******************************************/

    glutKeyboardFunc(keyboard);

    /* set up the logical graphics space */
    /*************************************/
    
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60, 1, 0.1, 110);
	glClearColor(0.0, 0.0, 0.0, 1.0);
	glMatrixMode(GL_MODELVIEW);

    /* assign the display function */
    /*******************************/

	glutDisplayFunc(display);

    /* assign the idle function */
    /****************************/

	glutIdleFunc(display);

    /* set up a timer to regularly update the scene */
    /************************************************/

	glutTimerFunc(5, update, 1);

    /* set everything going */
    /************************/
    
	glutMainLoop();
 
	return 0;  

}

