/****************************************************************	
* Name : Atul Nayak						*
* ped.cxx : Methods of class ped				*
****************************************************************/	     

#include "common.hxx"
#include "ped.hxx"

// constructor - reads in the ped.txt file
ped::ped()
{ 
	int tmp_noOfVertices;
	float tmp_vertex1;
	float tmp_vertex2;
	float tmp_vertex3;

	FILE* fp1 = fopen(pedfile,"r");	// open the file

	if (!fp1)
	{
		printf("Couldn't open file ped.txt\n");
		exit(1);
	}
	fscanf(fp1,"%d\n", &noOfPolygons);
	fscanf(fp1,"\n");

	fscanf(fp1,"%f %f %f", &colors[0], &colors[1], &colors[2]);
	fscanf(fp1,"\n");

	for (int i = 0; i < noOfPolygons; i++)
	{
		fscanf(fp1, "%d", &tmp_noOfVertices);
		noOfVertices.push_back(tmp_noOfVertices);
		fscanf(fp1,"\n");
		for (int k =0 ; k < tmp_noOfVertices; k++)
		{
			fscanf(fp1, "%f %f %f", &tmp_vertex1, &tmp_vertex2, &tmp_vertex3);
			vertices.push_back(tmp_vertex1);
			vertices.push_back(tmp_vertex2);
			vertices.push_back(tmp_vertex3);
			fscanf(fp1,"\n");
		}
	}
	fclose(fp1);
	orbTaken = FALSE;	// reset this to false in the beginning


}

//draw the pedestal in the specified position
void ped::drawPed(int x, int y)
{
	positionX = x;
	positionY = y;

	//apply transformations
	glPushMatrix();
		glTranslatef((x*5.0)+2.5,(y*5.0)+2.5,0.0);	
		glColor3f(colors[0], colors[1], colors[2]);
		int i =0;		// i keeps track of the no of coordinates
		for (int j = 0; j < noOfPolygons; j++)
		{
			glBegin(GL_LINE_LOOP);
				int g=0;	// g tracks no of corners in the polygon
				for ( ; g < noOfVertices[j]; i+=3, g++)
					glVertex3f(vertices[i], vertices[i+1], vertices[i + 2]);
			glEnd();
		}
	glPopMatrix();
	
}

// return positionX
int ped::getPositionX()
{
	return positionX;
}

// return positionX
int ped::getPositionY()
{
	return positionY;
}

// set the color to it's maximum intensity
void ped::brightestColor()
{
	// if the orb was taken then retain the red color 
	// and remove the green component	
	colors[0] = 1.0;
	if (orbTaken == FALSE)	
		colors[1] = 1.0;
	else
		colors[1] = 0.0;
	colors[2] = 0.0;		
}

// set the color to it's minimum intensity
void ped::darkestColor()
{
	colors[0] = 0.0;
	colors[1] = 0.0;		
	colors[2] = 0.0;		
}

// set the door color to an intermediate value specified
void ped::changeColor(float difference)
{
	// if the orb was taken then retain the red color 
	// and remove the green component	
	colors[0] = difference;
	if (orbTaken == FALSE)	
		colors[1] = difference;
	else
		colors[1] = 0.0;
	colors[2] = 0.0;		
}

// make the pedestal red in color
void ped::makeRed()
{
	colors[0] = 1.0;
	colors[1] = 0.0;
	colors[2] = 0.0;
}

// set the orbTaken flag
void ped::setOrbTaken()
{
	orbTaken = TRUE;
}

// destructor
ped::~ped()
{}
