/********************************************************
*	Name : Atul Nayak				*
*	tele.cxx : Defines the methods of teleporter.hxx*
********************************************************/
#include "common.hxx"
#include "tele.hxx"

// constructor  - reads the tele.txt file
teleporter::teleporter()
{ 
	int tmp_noOfVertices;			
	float tmp_vertex1;
	float tmp_vertex2;
	float tmp_vertex3;

	FILE* fp1 = fopen(telefile,"r");

	if (!fp1)
	{
		printf("Couldn't open file tele.txt\n");
		exit(1);
	}
	fscanf(fp1,"%d\n", &noOfPolygons);
	fscanf(fp1,"\n");

	fscanf(fp1,"%f %f %f", &colors[0], &colors[1], &colors[2]);
	fscanf(fp1,"\n");

	for (int i = 0; i < noOfPolygons; i++)
	{
		fscanf(fp1, "%d", &tmp_noOfVertices);
		noOfVertices.push_back(tmp_noOfVertices);
		fscanf(fp1,"\n");
		for (int k =0 ; k < tmp_noOfVertices; k++)
		{
			fscanf(fp1, "%f %f %f", &tmp_vertex1, &tmp_vertex2, &tmp_vertex3);
			vertices.push_back(tmp_vertex1);
			vertices.push_back(tmp_vertex2);
			vertices.push_back(tmp_vertex3);
			fscanf(fp1,"\n");
		}
	}
	fclose(fp1);
	angle = 0.0;	// set the angle to 0 at the start
}

// draw the teleporter at the location specified by x and y
void teleporter::drawTeleporter(int x, int y)
{
	positionX = x;
	positionY = y;
	angle++;		// increase the angle

	// apply transformations	
	glPushMatrix();
		glTranslatef((x*5.0)+2.5,(y*5.0)+2.5,0.0);	
		glRotatef(angle,0,0,1);
		glColor3f(colors[0], colors[1], colors[2]);
		int i =0;		// i keeps track of the no of coordinates
		for (int j = 0; j < noOfPolygons; j++)
		{
			glBegin(GL_LINE_LOOP);
				int g=0;	// g tracks no of corners in the polygon
				for ( ; g < noOfVertices[j]; i+=3, g++)
					glVertex3f(vertices[i], vertices[i+1], vertices[i + 2]);
			glEnd();
		}
	glPopMatrix();
}

// return positionX
int teleporter::getPositionX()
{
	return positionX;
}

// return positionY
int teleporter::getPositionY()
{
	return positionY;
}

// if the teleporter is within 10 feet make the teleporter bright
void teleporter::brightestColor()
{
	colors[0] = 0.0;
	colors[1] = 1.0;		
	colors[2] = 1.0;		
}

// if the teleporter is 110 feet away darken it
void teleporter::darkestColor()
{
	colors[0] = 0.0;
	colors[1] = 0.0;		
	colors[2] = 0.0;		
}

// change the color of the teleporter in proportion to the distance from the viewer
void teleporter::changeColor(float difference)
{
	colors[0] = 0.0;
	colors[1] = difference;		
	colors[2] = difference;		
}

// destructor
teleporter::~teleporter()
{}
